@@ -27,24 +27,31 @@ logger = logging.getLogger(__name__)
 
 
 def deep_format(obj, paramdict, allow_empty=False):
-    """Apply the paramdict via str.format() to all string objects found within
-       the supplied obj. Lists and dicts are traversed recursively."""
+    """Deep format configuration.
+
+    Apply the paramdict via str.format() to all string objects found within
+    the supplied obj. Lists and dicts are traversed recursively.
+    """
     # YAML serialisation was originally used to achieve this, but that places
     # limitations on the values in paramdict - the post-format result must
     # still be valid YAML (so substituting-in a string containing quotes, for
     # example, is problematic).
-    if hasattr(obj, 'format'):
+    if hasattr(obj, "format"):
         try:
             ret = CustomFormatter(allow_empty).format(obj, **paramdict)
         except KeyError as exc:
             missing_key = exc.args[0]
             desc = "%s parameter missing to format %s\nGiven:\n%s" % (
-                missing_key, obj, pformat(paramdict))
+                missing_key,
+                obj,
+                pformat(paramdict),
+            )
             raise JenkinsJobsException(desc)
         except Exception:
-            logging.error("Problem formatting with args:\nallow_empty:"
-                          "%s\nobj: %s\nparamdict: %s" %
-                          (allow_empty, obj, paramdict))
+            logging.error(
+                "Problem formatting with args:\nallow_empty:"
+                "%s\nobj: %s\nparamdict: %s" % (allow_empty, obj, paramdict)
+            )
             raise
 
     elif isinstance(obj, list):
@@ -55,24 +62,32 @@ def deep_format(obj, paramdict, allow_empty=False):
         ret = type(obj)()
         for item in obj:
             try:
-                ret[CustomFormatter(allow_empty).format(item, **paramdict)] = \
-                    deep_format(obj[item], paramdict, allow_empty)
+                ret[deep_format(item, paramdict, allow_empty)] = deep_format(
+                    obj[item], paramdict, allow_empty
+                )
             except KeyError as exc:
                 missing_key = exc.args[0]
                 desc = "%s parameter missing to format %s\nGiven:\n%s" % (
-                    missing_key, obj, pformat(paramdict))
+                    missing_key,
+                    obj,
+                    pformat(paramdict),
+                )
                 raise JenkinsJobsException(desc)
             except Exception:
-                logging.error("Problem formatting with args:\nallow_empty:"
-                              "%s\nobj: %s\nparamdict: %s" %
-                              (allow_empty, obj, paramdict))
+                logging.error(
+                    "Problem formatting with args:\nallow_empty:"
+                    "%s\nobj: %s\nparamdict: %s" % (allow_empty, obj, paramdict)
+                )
                 raise
     else:
         ret = obj
     if isinstance(ret, CustomLoader):
-        # If we have a CustomLoader here, we've lazily-loaded a template;
+        # If we have a CustomLoader here, we've lazily-loaded a template
+        # or rendered a template to a piece of YAML;
         # attempt to format it.
-        ret = deep_format(ret, paramdict, allow_empty=allow_empty)
+        ret = deep_format(
+            ret.get_object_to_format(), paramdict, allow_empty=allow_empty
+        )
     return ret
 
 
@@ -81,19 +96,26 @@ class CustomFormatter(Formatter):
     Custom formatter to allow non-existing key references when formatting a
     string
     """
-    _expr = '{({{)*(?:obj:)?(?P<key>\w+)(?:\|(?P<default>[\w\s]*))?}(}})*'
+
+    _expr = r"""
+        (?<!{){({{)*                # non-pair opening {
+        (?:obj:)?                   # obj:
+        (?P<key>\w+)                # key
+        (?:\|(?P<default>[^}]*))?   # default fallback
+        }(}})*(?!})                 # non-pair closing }
+    """
 
     def __init__(self, allow_empty=False):
         super(CustomFormatter, self).__init__()
         self.allow_empty = allow_empty
 
     def vformat(self, format_string, args, kwargs):
-        matcher = re.compile(self._expr)
+        matcher = re.compile(self._expr, re.VERBOSE)
 
         # special case of returning the object if the entire string
         # matches a single parameter
         try:
-            result = re.match('^%s$' % self._expr, format_string)
+            result = re.match("^%s$" % self._expr, format_string, re.VERBOSE)
         except TypeError:
             return format_string.format(**kwargs)
         if result is not None:
@@ -124,8 +146,7 @@ class CustomFormatter(Formatter):
         except KeyError:
             if self.allow_empty:
                 logger.debug(
-                    'Found uninitialized key %s, replaced with empty string',
-                    key
+                    "Found uninitialized key %s, replaced with empty string", key
                 )
-                return ''
+                return ""
             raise
